/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
*  contributor license agreements.  See the NOTICE file distributed with
*  this work for additional information regarding copyright ownership.
*  The ASF licenses this file to You under the Apache License, Version 2.0
*  (the "License"); you may not use this file except in compliance with
*  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.apache.yoko.orb.OB;

import java.util.logging.Level;

//
// IDL:orb.yoko.apache.org/OB/Logger:1.0
//
/**
 *
 * The Yoko message logger interface.
 *
 **/

public interface LoggerOperations
{
    //
    // IDL:orb.yoko.apache.org/OB/Logger/info:1.0
    //
    /**
     *
     * Log an informational message.
     *
     * @param msg The message.
     *
     **/

    void
    info(String msg);

    //
    // IDL:orb.yoko.apache.org/OB/Logger/error:1.0
    //
    /**
     *
     * Log an error message.
     *
     * @param msg The error message.
     *
     **/

    void
    error(String msg);

    //
    // IDL:orb.yoko.apache.org/OB/Logger/warning:1.0
    //
    /**
     *
     * Log a warning message.
     *
     * @param msg The warning message.
     *
     **/

    void
    warning(String msg);

    //
    // IDL:orb.yoko.apache.org/OB/Logger/warning:1.0
    //
    /**
     *
     * Log a debug message.
     *
     * @param msg The debug message.
     *
     **/

    void
    debug(String msg);

    //
    // IDL:orb.yoko.apache.org/OB/Logger/trace:1.0
    //
    /**
     *
     * Log a trace message.
     *
     * @param category The trace category.
     *
     * @param msg The trace message.
     *
     **/

    void
    trace(String category,
          String msg);

    /**
     * Log a message of the indicated level.
     *
     * @param level  The message level.
     * @param msg    The logged message.
     */
    void log(Level level, String msg);

    /**
     * Log a message of the indicated level.
     *
     * @param level  The message level.
     * @param msg    The logged message.
     * @param param  A single parameter object included with the message.
     */
    void log(Level level, String msg, Object param);

    /**
     * Log a message of the indicated level.
     *
     * @param level  The message level.
     * @param msg    The logged message.
     * @param params An array of parameter objects logged with the message.
     */
    void log(Level level, String msg, Object[] params);

    /**
     * Log a message of the indicated level.
     *
     * @param level  The message level.
     * @param msg    The logged message.
     * @param thrown An exception object included in the log.
     */
    void log(Level level, String msg, Throwable thrown);
}
