/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/

package org.apache.yoko.tools.processors.idl;

import javax.wsdl.Definition;

import antlr.collections.AST;

import org.apache.schemas.yoko.bindings.corba.TypeMappingType;

import org.apache.ws.commons.schema.XmlSchema;
import org.apache.ws.commons.schema.XmlSchemaCollection;

public class ModuleVisitor extends VisitorBase {

    private Definition definition;
    
    public ModuleVisitor(Scope scope,
                         Definition wsdlDefinition,
                         XmlSchemaCollection xmlSchemas,
                         XmlSchema xmlSchema,
                         TypeMappingType corbaTypeMap) {
        super(scope, xmlSchemas, xmlSchema, corbaTypeMap);
        definition = wsdlDefinition;
    }
    
    public void visit(AST node) {
        // <module> ::= "module" <identifier> "{" <definition>+ "}"
        
        AST identifierNode = node.getFirstChild();
        AST definitionNode = identifierNode.getNextSibling();
        
        while (definitionNode != null) {
            DefinitionVisitor definitionVisitor =
                new DefinitionVisitor(new Scope(getScope(), identifierNode),
                                      definition,
                                      schemas,
                                      schema,
                                      typeMap);

            definitionVisitor.visit(definitionNode);
            
            definitionNode = definitionNode.getNextSibling();
        }
        
    }
}
